#!/bin/bash

#Installation script for NE Terminal Config

_install() {
	#Install NE Terminal Config
	#$1 : the output path, if different of /
	#Share
	mkdir -pv "$1"/usr/share/ne-terminal-config/
	cp -v ./pixmaps/* "$1"/usr/share/ne-terminal-config/
	cp -v ./code/*.glade "$1"/usr/share/ne-terminal-config/
	#Bin
	mkdir -pv "$1"/usr/bin/
	cp -v ./code/ne-terminal-config.py "$1"/usr/bin/ne-terminal-config
	#Doc
	mkdir -pv "$1"/usr/share/doc/ne-terminal-config/
	cp -v README "$1"/usr/share/doc/ne-terminal-config/
	cp -v AUTHORS "$1"/usr/share/doc/ne-terminal-config/
	#locales
	for file in `find ./locales -name "*.po"` ; do {
		l10elang=`echo $file | sed -r 's#./locales/(.*).po#\1#g'`
		mkdir -pv "$1"/usr/share/locale/$l10elang/LC_MESSAGES/
		msgfmt "$file" -o "$1"/usr/share/locale/$l10elang/LC_MESSAGES/ne-terminal-config.mo
	} done
	#Copy this script for uninstalling, (not for --package)
	if [ "$1" == "" ] ; then {
		cp -v ./install.sh /usr/share/ne-terminal-config/
	} fi
}


_remove() {
	#Remove NE Terminal Config
	rm -rv /usr/share/ne-terminal-config
	rm -rv /usr/share/doc/ne-terminal-config
	rm -v /usr/bin/ne-terminal-config
	find /usr/share/locale/ -name ne-terminal-config.mo \
		-exec rm -v '{}' ';'
}


_locale() {
	#Make the transtation template
	mkdir -pv ./locales/
	xgettext -k_ -kN_ -o ./locales/ne-terminal-config.pot ./code/*.py ./code/*.glade
	for lcfile in $(find ./locales/ -name "*.po") ; do {
		echo -n "   * $lcfile"
		msgmerge --update $lcfile ./locales/ne-terminal-config.pot
	} done
}

#Force english
export LANG=c
#Go to the scrip directory
cd "${0%/*}" 1> /dev/null 2> /dev/null

#Head text
echo "NE Terminal Config - A configuration GUI for Nautilus Elementary Terminal."
echo

#Action do to
if [ "$1" == "--install" ] || [ "$1" == "-i" ] ; then {
	echo "Installing NE Terminal Config..."
	if [ "$(whoami)" == "root" ] ; then {
		_install
	} else {
		echo "E: Need to be root"
		exit 1
	} fi
} elif [ "$1" == "--package" ] || [ "$1" == "-p" ] ; then {
	echo "Packaging NE Terminal Config..."
	if [ -d "$2" ] ; then {
		_install "$2"
	} else {
		echo "E: '$2' is not a directory"
		exit 2
	} fi
} elif [ "$1" == "--remove" ] || [ "$1" == "-r" ] ; then {
	echo "Removing NE Terminal Config..."
	if [ "$(whoami)" == "root" ] ; then {
		_remove
	} else {
		echo "E: Need to be root"
		exit 1
	} fi
} elif [ "$1" == "--locale" ] || [ "$1" == "-l" ] ; then {
	echo "Extracting strings for the translation template..."
	if [ -x /usr/bin/xgettext ] ; then {
		_locale
	} else {
		echo "E: Can't find xgettext... gettex is installed ?"
		exit 3
	} fi
} else {
	echo "Arguments :"
	echo "  --install : install NE Terminal Config on your computer."
	echo "  --package <path> : install NE Terminal Config in <path> (packaging)."
	echo "  --remove : remove NE Terminal Config from your computer."
	echo "  --locale : extract strings to for the translation template."
} fi

exit 0


