#!/usr/bin/python
# -*- coding: UTF-8 -*-

   #########################################################################
 ##                                                                       ##
##           ┏┓╻┏━╸   ╺┳╸┏━╸┏━┓┏┳┓╻┏┓╻┏━┓╻     ┏━╸┏━┓┏┓╻┏━╸╻┏━╸           ##
##           ┃┗┫┣╸     ┃ ┣╸ ┣┳┛┃┃┃┃┃┗┫┣━┫┃     ┃  ┃ ┃┃┗┫┣╸ ┃┃╺┓           ##
##           ╹ ╹┗━╸    ╹ ┗━╸╹┗╸╹ ╹╹╹ ╹╹ ╹┗━╸   ┗━╸┗━┛╹ ╹╹  ╹┗━┛           ##
##        — A configuration GUI for Nautilus Elementary Terminal —        ##
##                                                                        ##
############################################################################
##                                                                        ##
## NE Terminal Config - A configuration GUI for NE Terminal               ##
##                                                                        ##
## Copyright (C) 2010  Fabien Loison (flo@flogisoft.com)                  ##
##                                                                        ##
## This program is free software: you can redistribute it and/or modify   ##
## it under the terms of the GNU General Public License as published by   ##
## the Free Software Foundation, either version 3 of the License, or      ##
## (at your option) any later version.                                    ##
##                                                                        ##
## This program is distributed in the hope that it will be useful,        ##
## but WITHOUT ANY WARRANTY; without even the implied warranty of         ##
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          ##
## GNU General Public License for more details.                           ##
##                                                                        ##
## You should have received a copy of the GNU General Public License      ##
## along with this program.  If not, see <http://www.gnu.org/licenses/>.  ##
##                                                                        ##
############################################################################
##                                                                        ##
## WEB SITE : http://software.flogisoft.com/nautilus-terminal/            ##
##                                                                       ##
#########################################################################


"""A configuration GUI for Nautilus Elementary Terminal"""

__author__ = "Fabien LOISON <flo@flogisoft.com>"
__version__ = "1.0"
__appname__ = "ne-terminal-config"


import os
import re
import gettext
gettext.install(__appname__)

import gtk
import pygtk
pygtk.require("2.0")
import pango
import vte
import gconf


#Paths
BASE_PATH = "/usr/share/ne-terminal-config"

if "DEVEL" in os.environ:
    GUI_FILE = "./ne-terminal-config.glade"
else:
    GUI_FILE = os.path.join(
            BASE_PATH,
            "ne-terminal-config.glade",
            )


#GConf
GCONF_KEY = "/apps/nautilus/ne_terminal/"


#Terminal
CURSOR_SHAPE = [
        _("Block"),
        _("Vertical bar"),
        _("Underlined"),
        ]

PREDEF_PALETTE = {
        'Tango': [
            "#000000", "#cc0000", "#4e9a06", "#c4a000",
            "#3465a4", "#75507b", "#06989a", "#d3d7cf",
            "#555753", "#ef2929", "#8ae234", "#fce94f",
            "#729fcf", "#ad7fa8", "#34e2e2", "#ffffff",
            ],
        'Linux console': [
            "#000000", "#aa0000", "#00aa00", "#aa5500",
            "#0000aa", "#aa00aa", "#00aaaa", "#aaaaaa",
            "#555555", "#ff5555", "#55ff55", "#ffff55",
            "#5555ff", "#ff55ff", "#55ffff", "#ffffff",
            ],
        'XTerm': [
            "#000000", "#cd0000", "#00cd00", "#cdcd00",
            "#1e90ff", "#cd00cd", "#00cdcd", "#e5e5e5",
            "#4c4c4c", "#ff0000", "#00ff00", "#ffff00",
            "#4682b4", "#ff00ff", "#00ffff", "#ffffff",
            ],
        'Rxvt': [
            "#000000", "#cd0000", "#00cd00", "#cdcd00",
            "#0000cd", "#cd00cd", "#00cdcd", "#faebd7",
            "#404040", "#ff0000", "#00ff00", "#ffff00",
            "#0000ff", "#ff00ff", "#00ffff", "#ffffff",
            ],
        'Evening': [
            "#333333", "#c75959", "#57af57", "#f5ce54",
            "#4b7bba", "#ca68ca", "#8bcfcf", "#a1aeaf",
            "#757575", "#ffa0a0", "#90ee90", "#ffff60",
            "#add8e6", "#ffbbff", "#e0ffff", "#ffffff",
            ],
        'Fruity': [
            "#111111", "#d1070d", "#4e9a06", "#fb660a",
            "#1066a9", "#ea037c", "#16afc6", "#8598a3",
            "#373737", "#d1070d", "#4e9a06", "#fb660a",
            "#1066a9", "#ea037c", "#16afc6", "#ffffff",
            ],
        }


class NETerminalConfig(object):
    """Preferences window for Nautilus Elementary Terminal."""

    def __init__(self):
        """The constructor."""
        self._gc = gconf.client_get_default()
        #GUI
        self.gui = gtk.Builder()
        self.gui.set_translation_domain(__appname__)
        self.gui.add_from_file(GUI_FILE)
        self.gui.connect_signals(self)
        #### winMain ####
        self.winMain = self.gui.get_object("winMain")
        #demoTerm
        self.demoTerm = vte.Terminal()
        self.demoTerm.set_size(50, 3)
        #Display something in the terminal
        self.demoTerm.feed("       _\|/_   zZ    ❭       ")
        self.demoTerm.feed("        ❭         _\|/_\n\033[1G")
        self.demoTerm.feed("       (- -) zZ      ❭  \_°< ")
        self.demoTerm.feed("Coin !  ❭         (O O)\n\033[1G")
        self.demoTerm.feed("---oOO--(_)--OOo---  ❭               ❭  ")
        self.demoTerm.feed("---oOO--(_)--OOo---\n\033[2J\033[1;1f")
        self.demoTerm.feed("[\033[40m \033[41m \033[42m \033[43m \033[44m ")
        self.demoTerm.feed("\033[45m \033[46m \033[47m \033[0m] ")
        self.demoTerm.feed("[\033[30m#\033[31m#\033[32m#\033[33m#\033[34m#")
        self.demoTerm.feed("\033[35m#\033[36m#\033[37m#\033[0m] ")
        self.demoTerm.feed("[\033[1;30m#\033[1;31m#\033[1;32m#\033[1;33m#")
        self.demoTerm.feed("\033[1;34m#\033[1;35m#\033[1;36m#\033[1;37m#")
        self.demoTerm.feed("\033[0m] [\033[0m#\033[0m\033[1m#\033[0m\033[4m#")
        self.demoTerm.feed("\033[0m\033[5m#\033[0m\033[7m#\033[0m]\n\033[1G")
        self.demoTerm.feed("NautilusElementary@TerminalConfig:~# ")
        self.demoTerm.show()
        #General
        self.cbShowScrollbar = self.gui.get_object("cbShowScrollbar")
        self.spbtnDefHeight = self.gui.get_object("spbtnDefHeight")
        self.entryCmd = self.gui.get_object("entryCmd")
        self.comboboxCursor = self.gui.get_object("comboboxCursor")
        self.lsstCursor = gtk.ListStore(str)
        self.comboboxCursor.set_model(self.lsstCursor)
        self.cellCursor = gtk.CellRendererText()
        self.comboboxCursor.pack_start(self.cellCursor, True)
        self.comboboxCursor.add_attribute(self.cellCursor, "text", 0)
        for shape in CURSOR_SHAPE:
            self.lsstCursor.append([shape])
        #Color
        self.comboboxPalette = self.gui.get_object("comboboxPalette")
        self.lsstPalette = gtk.ListStore(str)
        self.comboboxPalette.set_model(self.lsstPalette)
        self.cellPallette = gtk.CellRendererText()
        self.comboboxPalette.pack_start(self.cellPallette, True)
        self.comboboxPalette.add_attribute(self.cellPallette, "text", 0)
        self._palette_list = []
        for palette in PREDEF_PALETTE:
            self._palette_list.append(palette)
            self.lsstPalette.append([palette])
        self._palette_list.append("Custom")
        self.lsstPalette.append([_("Custom")])
        self.clbtnFg = self.gui.get_object("clbtnFg")
        self.clbtnBg = self.gui.get_object("clbtnBg")
        self.clbtnPalette = []
        for i in xrange(16):
            self.clbtnPalette.append(self.gui.get_object("clbtnPalette%i" % i))
            self.clbtnPalette[i].connect(
                    "color-set",
                    self.on_clbtnPalette_color_set,
                    i,
                    )
        #Font
        self.fontbtn = self.gui.get_object("fontbtn")
        self.cbAllowBold = self.gui.get_object("cbAllowBold")
        #vboxTerm
        self.alTerm = self.gui.get_object("alTerm")
        self.alTerm.add(self.demoTerm)
        #### winAbout ####
        self.winAbout = self.gui.get_object("winAbout")
        self.winAbout.connect("response", self.on_winAbout_response)
        self.winAbout.set_version(__version__)
        ####
        self._put_opt()

    #~~~~ winMAin ~~~~

    def on_btnClose_clicked(self, widget):
        gtk.main_quit()

    def on_btnAbout_clicked(self, widget):
        self.winAbout.show()

    def on_winMain_destroy(self, widget):
        gtk.main_quit()

    #~~~~ winMAin:General ~~~~

    def on_cbShowScrollbar_toggled(self, widget):
        key = GCONF_KEY + "scrollbar_show"
        self._gc.set_bool(key, widget.get_active())

    def on_spbtnDefHeight_value_changed(self, widget):
        key = GCONF_KEY + "default_height"
        self._gc.set_int(key, int(widget.get_value()))

    def on_entryCmd_changed(self, widget):
        key = GCONF_KEY + "shell"
        if os.path.isfile(widget.get_text()):
            self._gc.set_string(key, widget.get_text())

    def on_comboboxCursor_changed(self, widget):
        key = GCONF_KEY + "cursor_shape"
        self._gc.set_int(key, widget.get_active())
        self.demoTerm.set_cursor_shape(widget.get_active())

    #~~~~ winMain:Color ~~~~

    def on_clbtnFg_color_set(self, widget):
        key = GCONF_KEY + "text_color"
        self._gc.set_string(key, str(widget.get_color()))
        self._set_palette()

    def on_clbtnBg_color_set(self, widget):
        key = GCONF_KEY + "background_color"
        self._gc.set_string(key, str(widget.get_color()))
        self._set_palette()

    def on_comboboxPalette_changed(self, widget):
        #Get palette name
        key = GCONF_KEY + "colormap_custom_name"
        value = self._palette_list[widget.get_active()]
        self._gc.set_string(key, value)
        #Save palette
        key = GCONF_KEY + "colormap"
        value = self._get_palette(self._palette_list[widget.get_active()])
        self._gc.set_string(key, self._palette_to_string(value))
        #Set preview
        self._set_palette()

    def on_clbtnPalette_color_set(self, widget, index=0):
        """General method for on_color_set"""
        #Get the current palette
        key = GCONF_KEY + "colormap_custom_name"
        value = self._gc.get_string(key)
        palette = self._get_palette(value)
        #Modify the current palette
        palette[index] = str(widget.get_color())
        #Save custom palette
        key = GCONF_KEY + "colormap_custom"
        self._gc.set_string(key, self._palette_to_string(palette))
        #Save palette
        key = GCONF_KEY + "colormap"
        self._gc.set_string(key, self._palette_to_string(palette))
        #Set the palette name to Custom
        key = GCONF_KEY + "colormap_custom_name"
        self._gc.set_string(key, "Custom")
        self.comboboxPalette.set_active(self._palette_list.index("Custom"))
        #Set preview
        self._set_palette()

    #~~~~ winMain:Font ~~~~

    def on_fontbtn_font_set(self, widget):
        key = GCONF_KEY + "font"
        self._gc.set_string(key, self.fontbtn.get_font_name())
        font = pango.FontDescription(self.fontbtn.get_font_name())
        self.demoTerm.set_font(font)

    def on_cbAllowBold_toggled(self, widget):
        key = GCONF_KEY + "allow_bold_text"
        self._gc.set_bool(key, widget.get_active())
        self.demoTerm.set_allow_bold(widget.get_active())

    #~~~~ winAbout ~~~~

    def on_winAbout_delete_event(self, widget, event):
        self.winAbout.hide()
        return True #Don't delete

    def on_winAbout_response(self, widget, response):
        if response < 0:
            self.winAbout.hide()

    #~~~~

    def _put_opt(self):
        """Put options on the GUI"""
        #General
        key = GCONF_KEY + "scrollbar_show"
        value = self._gc.get_bool(key)
        self.cbShowScrollbar.set_active(value)

        key = GCONF_KEY + "default_height"
        value = self._gc.get_int(key)
        if value >= 50 and value <= 600:
            self.spbtnDefHeight.set_value(value)
        else:
            self.spbtnDefHeight.set_value(200)
            self._gc.set_int(key, 200)

        key = GCONF_KEY + "shell"
        value = self._gc.get_string(key)
        if os.path.isfile(value):
            self.entryCmd.set_text(value)
        else:
            self.entryCmd.set_text("/bin/bash")
            self._gc.set_string(key, "/bin/bash")

        key = GCONF_KEY + "cursor_shape"
        value = self._gc.get_int(key)
        if value in (0, 1, 2):
            self.comboboxCursor.set_active(value)
            self.demoTerm.set_cursor_shape(value)
        else:
            self.comboboxCursor.set_active(0)
            self.demoTerm.set_cursor_shape(0)
            self._gc.set_int(key, 0)

        #Color
        key = GCONF_KEY + "text_color"
        value = self._gc.get_string(key)
        if re.match(r"^#[0-9a-f]{3,}$", value.lower()):
            self.clbtnFg.set_color(gtk.gdk.Color(value))
        else:
            self.clbtnFg.set_color(gtk.gdk.Color("#000000"))
            self._gc.set_string(key, "#000000")

        key = GCONF_KEY + "background_color"
        value = self._gc.get_string(key)
        if re.match(r"^#[0-9a-f]{3,}$", value.lower()):
            self.clbtnBg.set_color(gtk.gdk.Color(value))
        else:
            self.clbtnBg.set_color(gtk.gdk.Color("#000000"))
            self._gc.set_string(key, "#000000")

        key = GCONF_KEY + "colormap_custom_name"
        value = self._gc.get_string(key)
        if not value in PREDEF_PALETTE and value != "Custom":
            value = "Tango"
        self.comboboxPalette.set_active(self._palette_list.index(value))
        self._set_palette()

        #Font
        key = GCONF_KEY + "font"
        value = self._gc.get_string(key)
        self.fontbtn.set_font_name(value)
        self.demoTerm.set_font(pango.FontDescription(value))

        key = GCONF_KEY + "allow_bold_text"
        value = self._gc.get_bool(key)
        self.cbAllowBold.set_active(value)
        self.demoTerm.set_allow_bold(value)

    def _set_palette(self):
        """Set palette in the GUI
        
        Arguments:
            * palette -- a list of 16 colors
        """
        key = GCONF_KEY + "colormap_custom_name"
        value = self._gc.get_string(key)
        palette = self._get_palette(value)
        fg = self.clbtnFg.get_color()
        bg = self.clbtnBg.get_color()
        palette_gdk = []
        for i in xrange(16):
            palette_gdk.append(gtk.gdk.Color(palette[i]))
            self.clbtnPalette[i].set_color(palette_gdk[i])
        self.demoTerm.set_colors(fg, bg, palette_gdk)

    def _get_palette(self, palette_name):
        """Return a list with all colors of the palette

        Arguments:
            * palette -- the palette name
        """
        if palette_name in PREDEF_PALETTE:
            return PREDEF_PALETTE[palette_name]
        elif palette_name == "Custom":
            key = GCONF_KEY + "colormap_custom"
            value = self._gc.get_string(key)
            if re.match(r"^(#[0-9a-f]{3,};){15}(#[0-9a-f]{3,})", value.lower()):
                return value.split(";")
            else:
                return PREDEF_PALETTE['Tango']
        else:
            return PREDEF_PALETTE['Tango']

    def _palette_to_string(self, palette):
        """Convert a palette (list of color) to a string

        Arguments:
            * palette -- a list of 16 color
        """
        result = ""
        for color in palette:
            result += "%s;" % color
        return result[:-1]


print("NE Terminal Config - %s" % __doc__)
GUI = NETerminalConfig()
gtk.main()


